package com.sun.xml.internal.ws.api.client;

import java.util.ArrayList;
import com.sun.xml.internal.ws.developer.WSBindingProvider;
import java.util.Collections;
import javax.xml.ws.WebServiceFeature;
import java.util.List;
import com.sun.xml.internal.ws.api.WSFeatureList;
import com.sun.istack.internal.Nullable;
import com.sun.istack.internal.NotNull;

public abstract class ServiceInterceptor
{
    public ServiceInterceptor() {
    }
    
    public List<WebServiceFeature> preCreateBinding(@NotNull WSPortInfo port, @Nullable Class<?> serviceEndpointInterface, @NotNull WSFeatureList defaultFeatures) {
        return Collections.emptyList();
    }
    
    public void postCreateProxy(@NotNull WSBindingProvider bp, @NotNull Class<?> serviceEndpointInterface) {
    }
    
    public void postCreateDispatch(@NotNull WSBindingProvider bp) {
    }
    
    public static ServiceInterceptor aggregate(final ServiceInterceptor... interceptors) {
        if (interceptors.length == 1) {
            return interceptors[0];
        }
        return new ServiceInterceptor() {
            @Override
            public List<WebServiceFeature> preCreateBinding(@NotNull WSPortInfo port, @Nullable Class<?> portInterface, @NotNull WSFeatureList defaultFeatures) {
                List<WebServiceFeature> r = new ArrayList();
                for (ServiceInterceptor si : interceptors) {
                    r.addAll(si.preCreateBinding(port, portInterface, defaultFeatures));
                }
                return r;
            }
            
            @Override
            public void postCreateProxy(@NotNull WSBindingProvider bp, @NotNull Class<?> serviceEndpointInterface) {
                for (ServiceInterceptor si : interceptors) {
                    si.postCreateProxy(bp, serviceEndpointInterface);
                }
            }
            
            @Override
            public void postCreateDispatch(@NotNull WSBindingProvider bp) {
                for (ServiceInterceptor si : interceptors) {
                    si.postCreateDispatch(bp);
                }
            }
        };
    }
}
